/*
 * irreco - Ir Remote Control
 * Copyright (C) 2008 Joni Kokko (t5kojo01@students.oamk.fi)
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "irreco_theme.h"

/*
 * Contains information of background
 */
 
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Prototypes                                                                 */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Construction & Destruction                                                 */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
IrrecoTheme *irreco_theme_new(IrrecoData * irreco_data, const char *folder_name)
{
	IrrecoTheme *self;
	IRRECO_ENTER
	
	self = g_slice_new0(IrrecoTheme);
	
	self->name = g_string_new(folder_name);
	self->backgrounds =  irreco_string_table_new(
				(GDestroyNotify)irreco_theme_bg_free, NULL);
	self->buttons =  irreco_string_table_new(
				(GDestroyNotify)irreco_theme_button_free, NULL);
	
	IRRECO_DEBUG("Themefolder: %s\n", folder_name);

	/*Get buttons*/
	{
		IrrecoDirForeachData button_styles;
		GString * directory = g_string_new("");	
	
		g_string_printf(directory, "%s/buttons/", folder_name);
		IRRECO_DEBUG("Directory = %s\n", directory->str);
		button_styles.directory = directory->str;
	
		button_styles.filesuffix = "bstyleconf";
		button_styles.user_data_1 = irreco_data;
		button_styles.user_data_2 = self;
	
		irreco_dir_foreach_subdirectories(&button_styles, 
				irreco_theme_read_button_keyfile_foreach);
		
		g_string_free(directory, TRUE);
		directory = NULL;
	}
	
	/*Get backgrounds*/
	{
		IrrecoDirForeachData bg_styles;
		GString * directory = g_string_new("");	
	
		g_string_printf(directory, "%s/bg/", folder_name);
		IRRECO_DEBUG("Directory = %s\n", directory->str);
		bg_styles.directory = directory->str;

		bg_styles.filesuffix = "";
		bg_styles.user_data_1 = irreco_data;
		bg_styles.user_data_2 = self;
	
		irreco_dir_foreach(&bg_styles,
				   irreco_theme_read_background_foreach);

		g_string_free(directory, TRUE);
		directory = NULL;
	}

	IRRECO_RETURN_PTR(self);
}

void irreco_theme_free(IrrecoTheme *self)
{
	IRRECO_ENTER
			
	g_assert(self != NULL);
	
	g_string_free(self->name, TRUE);
	self->name = NULL;
	
	irreco_string_table_free(self->backgrounds);
	self->backgrounds = NULL;

	irreco_string_table_free(self->buttons);
	self->buttons = NULL;

	g_slice_free(IrrecoTheme, self);
	self = NULL;

	IRRECO_RETURN
}

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Private Functions                                                          */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Functions                                                                  */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

void irreco_theme_print(IrrecoTheme *self)
{
	IRRECO_ENTER

	IRRECO_PRINTF("Theme: %s \n", self->name->str);
	irreco_string_table_print(self->backgrounds);
	irreco_string_table_print(self->buttons);
	
	#if 0
	IRRECO_STRING_TABLE_FOREACH_DATA(self->backgrounds, void *, theme_bg)
		irreco_theme_button_print(theme_bg);
	IRRECO_STRING_TABLE_FOREACH_END

	IRRECO_STRING_TABLE_FOREACH_DATA(self->buttons, void *, theme_button)
		irreco_theme_button_print(theme_button);
	IRRECO_STRING_TABLE_FOREACH_END
	#endif

	IRRECO_RETURN
}

void irreco_theme_read_button_keyfile_foreach(IrrecoDirForeachData * dir_data)
{
	IrrecoData	*irreco_data	= (IrrecoData*) dir_data->user_data_1;
	IrrecoTheme	*self		= (IrrecoTheme*) dir_data->user_data_2;
	IrrecoKeyFile	*keyfile;
	IrrecoButtonStyle *style;
	IrrecoThemeButton *theme_button;
	IRRECO_ENTER

	keyfile = irreco_keyfile_create(dir_data->directory, 
					dir_data->filepath, 
					"button-style");
	if (keyfile == NULL) goto end;
	
	style = irreco_button_style_create(irreco_data, NULL, NULL,
					   NULL, TRUE, FALSE);
	
	/* Required fields. */			
	if (!irreco_keyfile_get_str(keyfile, "name", &style->name) ||
	    !irreco_keyfile_get_path(keyfile, "up", &style->up)) {
		IRRECO_PRINTF("Could not read style from group \"%s\"\n", 
			      keyfile->group);
		irreco_button_style_destroy(style);
		IRRECO_RETURN
	}
	
	/* Optional fields. */
	irreco_keyfile_get_path(keyfile, "down", &style->down);
	irreco_keyfile_get_bool(keyfile, "allow-text", &style->allow_text);
	irreco_keyfile_get_str(keyfile, "text-format-up", &style->text_format_up);
	irreco_keyfile_get_str(keyfile, "text-format-down", 
			       &style->text_format_down);
	irreco_keyfile_get_int(keyfile, "text-padding", &style->text_padding);
	
	/* Attempt to add the style to the style array. */
	#if 1
	if (!irreco_string_table_add(irreco_data->button_style_array,
				     style->name, style)) {
		IRRECO_ERROR("Dublicate style name \"%s\"\n", style->name);
		irreco_button_style_destroy(style);
	}
	#endif
	
	theme_button = irreco_theme_button_new();
	irreco_theme_button_set(theme_button,
				style->name, style->allow_text,
				style->up, style->down,
				style->text_format_up, style->text_format_down,
				style->text_padding);
	
	irreco_string_table_add(self->buttons, style->name, theme_button);
	
	irreco_theme_button_print(theme_button);

	irreco_keyfile_destroy(keyfile);
	
	end:

	IRRECO_RETURN
}



void irreco_theme_read_background_foreach(IrrecoDirForeachData * dir_data)
{
	IrrecoTheme	*self = (IrrecoTheme*) dir_data->user_data_2;
	IrrecoThemeBg	*bg;
	IRRECO_ENTER
	
	bg = irreco_theme_bg_new();
	
	irreco_theme_bg_set(bg, dir_data->filename, dir_data->filepath);
	
	irreco_string_table_add(self->backgrounds, dir_data->filename, bg);
	
	irreco_theme_bg_print(bg);

	IRRECO_RETURN
}

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Events and Callbacks                                                       */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

